#!/usr/bin/env python
# coding=utf-8
#-----------------------------------------------------------------------
# Copyright © 2014-2018 Tormach® Inc. All rights reserved.
# License: GPL Version 2
#
# Modified by David Loomes
# Shared with permission by Saunders Machine Works & NYC CNC
# 09/06/2022
#-----------------------------------------------------------------------

# if this file exists in ~/gcode/python/ui_hooks.py the UI will import it

# PathPilot constants
import constants

from threading import Thread
import time

import linuxcnc
#s.block_delete
#halCmd = linuxcnc.command()
#c.set_block_delete(int)

class ui_hooks():

    def __init__(self, linuxcnc_command, error_handler, ver_list, digital_output_offset=0):
        self.halCommand = linuxcnc_command
        self.halStatus = linuxcnc.stat()
        self.error_handler = error_handler
        self.ver_list = ver_list
        # digital_output_offset is for the lathe config where USBIO outputs start at '5' and mill config they start at output '0'
        self.digital_output_offset = digital_output_offset

        # can test PathPilot version if needed
        # ver_list = [major int, minor int, build int, suffix_string ]
        self.error_handler.write('PathPilot version: %s.%s.%s' % (self.ver_list[0], self.ver_list[1], self.ver_list[2]), constants.ALARM_LEVEL_QUIET)

        # log that we were found
        self.error_handler.write('ui_hooks:__init__() called', constants.ALARM_LEVEL_DEBUG)


        # start the worker thread
        self.probeMonitorthread = Thread(target=self.monitorProbe)
        self.probeMonitorthread.daemon = True
        self.probeMonitorthread.start()

    def __clear_usbio_outputs(self, reason, output_pin_tuple):
        self.error_handler.write('%sClearing USBIO outputs %s' % (reason, output_pin_tuple, ), constants.ALARM_LEVEL_QUIET)
        # self.digital_output_offset is for the lathe config where USBIO outputs start at '5' and mill config they start at output '0'
        for pin_number in output_pin_tuple:
            self.halCommand.set_digital_output(self.digital_output_offset + pin_number, 0)
            self.halCommand.wait_complete()

    def stop_button(self):
        self.__clear_usbio_outputs('Stop button. ', (0, 1, 2, 3))

    def reset_button(self):
        self.__clear_usbio_outputs('Reset button. ', (0, 1, 2, 3))

    def estop_event(self):
        self.__clear_usbio_outputs('E-Stop event. ', (0, 1, 2, 3))


    def monitorProbe(self):
        self.error_handler.write('Probe/spindle guard is running', constants.ALARM_LEVEL_QUIET)
        while True:
            # make sure hal status info is up to date
            self.halStatus.poll()

            # is the probe in the spindle
            if self.halStatus.tool_in_spindle == 99:
                # current tool is the probe, check for spindle start
                if self.halStatus.spindle_enabled:
                    # issue an abort if starting the spindle with the probe loaded
                    self.halCommand.abort()
                    self.error_handler.write('Spindle halted - probe in spindle')

                # with the probe in the spindle, short sleep before checking again
                time.sleep(0.1)
            else:
                # tool in spindle is not the probe, so sleep for longer before checkng again
                time.sleep(1.0)

